/*
    datepickr - pick your date not your nose
    Copyright (c) 2012
*/

var datepickr = (function() {
	var datepickrs = [],
	currentDate = new Date(),
	date = {
		current: {
			year: function() {
				return currentDate.getFullYear();
			},
			month: {
				integer: function() {
					return currentDate.getMonth();
				},
				string: function(full, months) {
					var date = currentDate.getMonth();
					return monthToStr(date, full, months);
				}
			},
			day: function() {
				return currentDate.getDate();
			}
		},
		month: {
			string: function(full, currentMonthView, months) {
				var date = currentMonthView;
				return monthToStr(date, full, months);
			},
			numDays: function(currentMonthView, currentYearView) {
				/* checks to see if february is a leap year otherwise return the respective # of days */
				return (currentMonthView == 1 && !(currentYearView & 3) && (currentYearView % 1e2 || !(currentYearView % 4e2))) ? 29 : daysInMonth[currentMonthView];
			}
		}
	},
	daysInMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31],
	buildCache = [],
	handlers = {
		calendarClick: function(e) {
			if(e.target.className) {
				switch(e.target.className) {
					case 'prev-month':
					case 'prevMonth':
					case 'ico ico-go-to-arrow-left':
						this.currentMonthView--;
						if(this.currentMonthView < 0) {
							this.currentYearView--;
							this.currentMonthView = 11;
						}
						rebuildCalendar.call(this);
					break;
					case 'next-month':
					case 'nextMonth':
					case 'ico ico-go-to-arrow-right':
						this.currentMonthView++;
						if(this.currentMonthView > 11) {
							this.currentYearView++;
							this.currentMonthView = 0;
						}
						rebuildCalendar.call(this);
					break;
					case 'day':
					case 'day circled':
						this.container.value = formatDate(new Date(this.currentYearView, this.currentMonthView, e.target.innerHTML).getTime(), this.config);
						this.close();
					break;
				}
			}
		},
		documentClick: function(e) {
			if(e.target != this.element && e.target != this.calendar) {
				var parentNode = e.target.parentNode;
				if(parentNode != this.calender) {
					while(parentNode != this.calendar) {
						parentNode = parentNode.parentNode;
						if(parentNode == null) {
							this.close();
							break;
						}
					}
				}
			}
		}
	};

	function formatDate(milliseconds, config) {
		var formattedDate = '',
		dateObj = new Date(milliseconds),
		format = {
			d: function() {
				var day = format.j();
				return (day < 10) ? '0' + day : day;
			},
			D: function() {
				return config.weekdays[format.w()].substring(0, 3);
			},
			j: function() {
				return dateObj.getDate();
			},
			l: function() {
				return config.weekdays[format.w()];
			},
			S: function() {
				return config.suffix[format.j()] || config.defaultSuffix;
			},
			w: function() {
				return dateObj.getDay();
			},
			F: function() {
				return monthToStr(dateObj.getMonth(), true, config.months);
			},
			m: function() {
				return (format.n() < 10) ? '0' + format.n() : format.n();
			},
			M: function() {
				return monthToStr(dateObj.getMonth(), false, config.months);
			},
			n: function() {
				return parseInt(dateObj.getMonth()) + 1;
			},
			Y: function() {
				return dateObj.getFullYear();
			},
			y: function() {
				return format.Y().toString().substring(2, 4);
			}
		},
		formatPieces = config.dateFormat.split('');

		foreach(formatPieces, function(formatPiece, index) {
			if(format[formatPiece] && formatPieces[index - 1] != '\\') {
				formattedDate += format[formatPiece]();
			} else {
				if(formatPiece != '\\') {
					formattedDate += formatPiece;
				}
			}
		});

		return formattedDate;
	}

	function foreach(items, callback) {
		var i = 0, x = items.length;
		for(i; i < x; i++) {
			if(callback(items[i], i) === false) {
				break;
			}
		}
	}

	function addEvent(element, eventType, callback) {
		if(element.addEventListener) {
			element.addEventListener(eventType, callback, false);
		} else if(element.attachEvent) {
			var fixedCallback = function(e) {
				e = e || window.event;
				e.preventDefault = (function(e) {
					return function() { e.returnValue = false; }
				})(e);
				e.stopPropagation = (function(e) {
					return function() { e.cancelBubble = true; }
				})(e);
				e.target = e.srcElement;
				callback.call(element, e);
			};
			element.attachEvent('on' + eventType, fixedCallback);
		}
	}

	function removeEvent(element, eventType, callback) {
		if(element.removeEventListener) {
			element.removeEventListener(eventType, callback, false);
		} else if(element.detachEvent) {
			element.detachEvent('on' + eventType, callback);
		}
	}

	function buildNode(nodeName, attributes, content, currentMonthView, currentYearView ) {
		var element, clsn, idx;

		if(!(nodeName in buildCache)) {
			buildCache[nodeName] = document.createElement(nodeName);
		}

		element = buildCache[nodeName].cloneNode(false);

		if(attributes != null) {
			for(var attribute in attributes) {
				element[attribute] = attributes[attribute];
			}
		}

		if ( nodeName == 'td' ) {
			if ( content != '&nbsp;' ) {
				element.setAttribute('data-date', currentYearView + '-'+ currentMonthView + '-' + parseInt(content));
			}
		}

		if(content != null) {
			if(typeof(content) == 'object') {
				element.appendChild(content);
			} else {
				element.innerHTML = content;
			}
		}

		return element;
	}

	function monthToStr(date, full, months) {
		return ((full == true) ? months[date] : ((months[date].length > 3) ? months[date].substring(0, 3) : months[date]));
	}

	function isToday(day, currentMonthView, currentYearView) {
		return day == date.current.day() && currentMonthView == date.current.month.integer() && currentYearView == date.current.year();
	}

	function buildWeekdays(weekdays) {
		var weekdayHtml = document.createDocumentFragment();
		foreach(weekdays, function(weekday) {
			weekdayHtml.appendChild(buildNode('th', {className : 'cal'}, weekday.substring(0, 2)));
		});
		return weekdayHtml;
	}

	function rebuildCalendar() {
		while(this.calendarBody.hasChildNodes()){
			this.calendarBody.removeChild(this.calendarBody.lastChild);
		}

		var firstOfMonth = new Date(this.currentYearView, this.currentMonthView, 1).getDay(),
		numDays = date.month.numDays(this.currentMonthView, this.currentYearView);

		this.currentMonth.innerHTML = date.month.string(this.config.fullCurrentMonth, this.currentMonthView, this.config.months) + ' ' + this.currentYearView;
		this.calendarBody.appendChild(buildDays(firstOfMonth, numDays, this.currentMonthView, this.currentYearView));

		if (tbb) {var days = document.getElementsByClassName('day'); for(var i=0;i<days.length;i++)days[i].addEventListener("click",function(a){if (!a.target.classList.contains('restricted')) tbb.changedate(a.target.getAttribute("data-date"))}),days[i].addEventListener("mouseover",function(a){a.target.classList.toggle('circled');}),days[i].addEventListener("mouseout",function(a){a.target.classList.toggle('circled');});}
	}

	function buildCurrentMonth(config, currentMonthView, currentYearView, months) {
		return buildNode('span', { className: 'current-month' }, date.month.string(config.fullCurrentMonth, currentMonthView, months) + ' ' + currentYearView);
	}

	function buildMonths(config, currentMonthView, currentYearView) {
		var months = buildNode('div', { className: 'months' }),
		prevMonth = buildNode('span', { className: 'prev-month' }, buildNode('span', { className: 'ico ico-go-to-arrow-left' })),
		nextMonth = buildNode('span', { className: 'next-month' }, buildNode('span', { className: 'ico ico-go-to-arrow-right' }));
		months.appendChild(prevMonth);
		months.appendChild(nextMonth);

		return months;
	}

	function buildDays(firstOfMonth, numDays, currentMonthView, currentYearView) {
		var calendarBody = document.createDocumentFragment(),
		row = buildNode('tr'),
		dayCount = 0, i, k = 1, cName;

		/* switch to monday in case week day should start from Monday */
		if (DateConfig._startCal && DateConfig._startCal == 1) {
			firstOfMonth = (firstOfMonth+6)%7;
		}

		/* print out previous month's "days" */
		for(i = 1; i <= firstOfMonth; i++) {
			row.appendChild(buildNode('td', null, '&nbsp;'));
			dayCount++;
			k++;
		}

		for(i = 1; i <= numDays; i++) {

			cName = 'day'

			/* if we have reached the end of a week, wrap to the next line */
			if(dayCount == 7) {
				calendarBody.appendChild(row);
				row = buildNode('tr');
				dayCount = 0;
			}

			var todayClassName = isToday(i, currentMonthView, currentYearView) ? { className: 'today' } : null;

			/* mark unavailable days */
			if (DateConfig._restricted) {

				var t = new Date(currentYearView, currentMonthView, parseInt(i) + 1);
				var d = t.toJSON().split('T')[0];

				if (DateConfig._restricted.indexOf(t.toISOString().split('T')[0]) !== -1) {
					cName = 'restricted day';
				}

				if (DateConfig._startCal && DateConfig._startCal == 1) {
					idx = (k%7) == 0 ? '7' : (k%7);
				} else {
					idx = ((k+6)%7) == 0 ? '7' : ((k+6)%7);
				}

				if (DateConfig._restricted.indexOf(idx.toString()) !== -1) {
    				if (DateConfig._allowed.indexOf(d) === -1) {
						cName = 'restricted day';
    				}
				}
			}
			/* mark unavailable days */

			row.appendChild(buildNode('td', { className: cName }, i, currentMonthView, currentYearView ));

			dayCount++;k++;
		}

		/* if we haven't finished at the end of the week, start writing out the "days" for the next month */
		for(i = 1; i <= (7 - dayCount); i++) {
			row.appendChild(buildNode('td', null, '&nbsp;'));
		}

		calendarBody.appendChild(row);

		return calendarBody;
	}

	function buildCalendar() {
		var firstOfMonth = new Date(this.currentYearView, this.currentMonthView, 1).getDay(),
		numDays = date.month.numDays(this.currentMonthView, this.currentYearView),
		self = this;

		var inputLeft = inputTop = 0,
		obj = this.element;

		var calendarContainer = document.createElement('div');
		calendarContainer.className += ' calendar';
		calendarContainer.id = 'mycal';
		calendarContainer.style.cssText = 'display: none; position: absolute; top:35px; z-index: 1100;';

		obj.parentNode.appendChild(calendarContainer);

		this.currentMonth = buildCurrentMonth(this.config, this.currentMonthView, this.currentYearView, this.config.months)
		var months = buildMonths(this.config, this.currentMonthView, this.currentYearView);
		months.appendChild(this.currentMonth);

		var calendar = buildNode('table',  { className: 'table calendars' } , buildNode('thead', null, buildNode('tr', { className: 'weekdays' }, buildWeekdays(this.config.weekdays))));
		this.calendarBody = buildNode('tbody');
		this.calendarBody.appendChild(buildDays(firstOfMonth, numDays, this.currentMonthView, this.currentYearView));
		calendar.appendChild(this.calendarBody);

		calendarContainer.appendChild(months);
		calendarContainer.appendChild(calendar);

		//document.body.appendChild(calendarContainer);

		addEvent(calendarContainer, 'click', function(e) { handlers.calendarClick.call(self, e); });

		return calendarContainer;
	}


	/*
		d m Y - 03 12 2014
		d-m-Y - 03-12-2014
		Y-m-d - 2014-12-03
		d/m/Y - 03/12/2014
		d.m.Y - 03.12.2014
		d M Y - 03 Dec 2014
		F d, Y - December 03, 2014
		j.n.Y - 3.12.2014
		d-m-y - 03-12-14
	*/
	return function(elementId, containerId, userConfig) {
		var self = this;

		this.element = document.getElementById(elementId);
		this.container = document.getElementById(containerId);

		this.config = {
			fullCurrentMonth: true,
			dateFormat: 'Y-m-d',
			weekdays: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
			months: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
			suffix: { 1: 'st', 2: 'nd', 3: 'rd', 21: 'st', 22: 'nd', 23: 'rd', 31: 'st' },
			defaultSuffix: 'th',
			currentTime: currentDate.getTime(),
			_startCal:0,
		};
		this.currentYearView = date.current.year();
		this.currentMonthView = date.current.month.integer();

		if(userConfig) {
			for(var key in userConfig) {
				if(this.config.hasOwnProperty(key)) {
					this.config[key] = userConfig[key];
				}
			}
		}

		this.documentClick = function(e) { handlers.documentClick.call(self, e); }

		this.open = function(e) {
			addEvent(document, 'click', self.documentClick);

			foreach(datepickrs, function(datepickr) {
				if(datepickr != self) {
					datepickr.close();
				}
			});

			self.calendar.style.display = 'block';
		}

		this.close = function() {
			removeEvent(document, 'click', self.documentClick);
			self.calendar.style.display = 'none';
		}

		this.calendar = buildCalendar.call(this);

		datepickrs.push(this);

		if(this.element.nodeName == 'INPUT') {
			addEvent(this.element, 'focus', this.open);
		} else {
			addEvent(this.element, 'click', this.open);
		}

		this.container.value = this.config.currentTime === 0 ? '' : formatDate( this.config.currentTime, this.config );
		document.getElementById('timestamp').value = this.config.currentTime;
	}
})();
