<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

use \Joomla\CMS\Date\Date;

class TbDisplay
{

    /*
     * method to display properly formatted time
     * @params $time (int) time value in seconds
     * @params $timeformat (string) restaurant time format param value
     *
     * return (string) hour formatted according to settings, if no settings default to 12 hours format
     */
    public static function showTime( $time, $timeformat = '12' )
    {
	  if ( !$time )
	  {
		return false;
	  }

	  $hour = '';

	  $hours = floor($time / 3600);
	  $minutes = ($time-($hours*3600))/60;
	  $minutes = ( strlen( $minutes ) < 2 ) ? '0' . $minutes : $minutes;

        $hours = $hours%24;
	  switch ( $timeformat )
	  {
		case '24' :
		    $hour = $hours . ':' . $minutes;
		break;

		case '12':
		default:
			if ( $hours == 12 )
			{
				$hour = '12:' . $minutes . ' PM';
			}
			else
			{
				$hour = ( $hours > 12 ) ? ($hours - 12) . ':' . $minutes . ' PM' : $hours . ':' . $minutes . ' AM';
			}
		break;
	  }
	  return $hour;
    }


    public static function loadMenu()
    {
    	$path = JPATH_COMPONENT . DIRECTORY_SEPARATOR . 'helpers' . DIRECTORY_SEPARATOR  . 'default_menu.php';
    	require_once( $path );
    }



    /**
     * generate the html view of tables in the management area
     *
     * @param [array] $possibilities
     * @return string
     */
    public static function viewManagTables( $possibilities )
    {
		$html = '<div class="form-group">';
		$html .= '<label class="control-label col-md-2">'. JText::_('COM_TABLEBOOKING_SEARCH_AVAILABLE_TABLES') .'</label>';
		$html .= '<div class="controls cold-md-10">';

    	foreach ( $possibilities as $key=>$possible )
    	{
    		$display = new stdclass;
    		$display->text = '';
    		$display->js = '';


    		if (is_array($possible))
    		{
    			foreach ( $possible as $table )
    			{
    				$display->text .= ( $display->text != '' ) ? ' + <i class="ico ico-tables"></i> ' . $table->name : '<i class="ico ico-tables"></i> ' . $table->name;
    				$display->js .= ( $display->js != '' ) ? ',' . $table->id : $table->id;
    			}
    		}
    		else
    		{
    			$display->text = '<i class="ico ico-tables"></i> '. $possible->name;
    			$display->js = $possible->id;
    		}
    		$html .= '<span id="tb'.$key.'" class="label label-primary pull-left" onclick="tbb.selectTable(\'tb'.$key.'\', \''. $display->js .'\');">' . $display->text . '</span>';
    	}

    	$html .= '</div>';
    	$html .= '</div>';

    	return $html;
    }


    /**
     * transform a date into a timestamp
     */
    public static function strToTime( $date )
    {
    	date_default_timezone_set('GMT');
    	return strtotime( $date );
    }


    /**
     *
     * display date in the chosen format
     *
     * @param  [type] $date   [date/timestamp]
     * @param  string $format [default to YYYY-mm-dd]
     * @return [string]
     */
    public static function showDate($date, $format = 'Y-m-d')
    {
        if(preg_match('/[\D]{1,}/', $date))
        {
            $timestamp = self::strToTime($date);
        }

        date_default_timezone_set('GMT');
        $jdate = new Date($date);
        return $jdate->format($format, false, true);
        //return date($format, $timestamp);
    }


    /**
     *
     * get some default hours in case no schedule is set
     *
     * @param  	object	$restaurant
     * @param 	boolean $translated [if to display hours as typical hour set to true, default is false so the hours will be display as number of seconds]
     * @return [array]
     */
    public static function defaultHours( $restaurant, $translated = false )
    {
        $params = $restaurant->getParams();
        $hours = array();
        $start = 7*3600;
        $end = 24*3600;
        for ( $i = $start; $i <= $end ; $i+= 300 )
        {
            $hours[$i] = $translated ? self::showTime($i, $params->hourformat) : $i;
        }
        return $hours;
    }



    /**
     * generate a table row contained by the schedule
     *
     * @param  [array]  array containg the new row details
     * @return string
     */
    public static function newRow($data)
    {
        $html = '<tr>';
        $html .= '<td>' . self::showTime($data['start']) . '</td>';
        $html .= '<td>' . self::showTime($data['end']) . '</td>';
        $html .= '<td><a href="javascript:void(0);" onclick=\'tb.deleteInterval(this, "' . $data['id'] . '", "' . $data['start'] . '", "' . $data['end'] . '");return false;\'><i class="delete ico ico-close"></i></a></td>';
        $html .= '</tr>';
        return $html;
    }


    /**
     * return html icon based on booking status
     * @param  [int] $status  unconfirmed(0),confirmed(1),canceled(2),seated(3)
     * @return [string]
     */
    public static function showStatusIcon($status)
    {
        switch ($status) {
            case 0:
                $icon = 'ico-question';
                break;

            case 1:
                $icon = 'ico-accept';
                break;

            case 2:
                $icon = 'ico-close';
                break;

            case 3:
                $icon = 'ico-restaurant';
                break;

            default:
                 $icon = 'ico-accept';
                break;
        }
        return '<i class="ico ' . $icon . '"></i>';
    }

    /**
     * @param  	[array] $tables 				[array of tables]
     * @return 	[string]
     */
    public static function getTablesArrangement($tables)
    {
        $arrangement = new stdclass;
        $arrangement->unassigned = [];
        $arrangement->assigned = [];

        $max = ceil(count($tables) * 1.25);
        $arrangement->axes = new stdclass;
        $arrangement->axes->fixed = [$max, $max];
        $arrangement->axes->relative = self::getAxesRelative($tables);

        for ($i = 0; $i <= $arrangement->axes->fixed[0]; $i++) {
            for ($j = 0; $j <= $arrangement->axes->fixed[1]; $j++) {
                $arrangement->assigned[$i][$j] = null;
            }
        }

        if ($tables)
        {
            foreach ($tables as $table)
            {
                if (!empty($table->params)){
                    $coordinates = explode('x', $table->params);
                    $arrangement->assigned[$coordinates[0]][$coordinates[1]] = $table;
                } else {
                    $arrangement->unassigned[] = $table;
                }
            }
        }
        return $arrangement;
    }

    protected static function getAxesRelative($tables)
    {
        $x = 0; $y = 0;

        if (!empty($tables)) {
            foreach ($tables as $table) {
                if (!empty($table->params)) {
                    $coordinates = explode('x', $table->params);
                    $x = ($coordinates[0] > $x) ? $coordinates[0] : $x;
                    $y = ($coordinates[1] > $y) ? $coordinates[1] : $y;
                }
            }
        }
        return [$x, $y];
    }

    public static function showTablesArrangement($tables, $possible, $selected)
    {
        $arrangement = self::getTablesArrangement($tables);
        $available = self::filterId($possible, array());
        $html = '<div id="arrangements">';
        $html .= '<div id="assigned">';

        for ($i = 0; $i<=$arrangement->axes->relative[0]; $i++) {
            $html .= '<div class="line">';
            for ($j = 0; $j<=$arrangement->axes->relative[1]; $j++) {
                $html .= '<div class="dropzone">';
                if(!is_null($item = $arrangement->assigned[$i][$j])) {
                $av = in_array($item->id, $available) ?
                                            'available' : 'unavailable';
                $click = $av == 'available' ?
                        'onclick="tbb.selectTable(this); "' : ' ';
                $marked = $selected == $item->id ? ' selected ' : ' ';
                $html .= '<div '. $click .
                        'data-id="'.$item->id.'" '.
                        'data-places="' . $item->max_places . '" '.
                        'class="draggable '.$marked.$av.' shape-'.$item->shape.'">'
                        . '<i class="ico ico-accept"></i>'.$item->name.'</div>';
                }
                $html .= '</div>';
            }
            $html .= '</div>';
        }

        $html .= '</div>';
        $html .= '</div>';
        return $html;
    }

    protected static function filterId($possible, $stored)
    {
        if (!empty($possible)) {
            foreach ($possible as $item) {
                if (is_array($item)) {
                    return self::filterId($item, $stored);
                } else {
                    $stored[] = $item->id;
                }
            }
        }

        return $stored;
    }

    public static function showBookingForm($form)
    {
        $html = '';

        foreach ($form->getFieldset() as $field)
        {
            $html .= '<div class="form-group">';
            $html .= '<label class="control-label col-md-2">'.$field->label.'</label>';
            $html .= '<div class="controls cold-md-10">' . $field->input . '</div>';
            $html .= '</div>';
        }

        return $html;
    }


    /*
     * return properly formated message subject/body to be sent to client/admin
     *
     * @params [string] $str 	subject or body message
     * @params [array] $data	reservation details to be used to replace tags
     * @params [object] $restaurant_params		selected restaurant params, to properly format date and hour
     */
    public static function stripNotificationsTags($str,$data,$restaurant_params)
	{
		// if the subject or body is a language string
		// useful if we want the subject or the body to be multilingual
		if (preg_match('/^[A-Z_]{1,}$/',$str,$matches))
		{
			$str = JText::_($matches[0]);
		}
		$tables = TbRestaurant::getInstance($data['restaurant'])
						->getTableWithIds(json_decode($data['table']));

        $table_names = array_reduce($tables, function($carry, $table) {
            if ($carry) {
                return $carry . ', ' . $table->name;
            } else {
                return $table->name;
            }
		}, '');

		$needle = array('%date%', '%from%', '%to%', '%table%', '%places%', '%name%', '%email%', '%phone%', '%comments%', '%confirmation_url%', '%cancelation_url%');
		$replacement = array(
			self::showDate($data['bookdate'], $restaurant_params->dateformat),
			self::showTime($data['bookstart'], $restaurant_params->hourformat),
			self::showTime($data['bookend'], $restaurant_params->hourformat),
			$table_names,
			(int) $data['places'],
			$data['name'],
			$data['email'],
			$data['phone'],
			$data['comments'],
			JUri::root() . 'index.php?option=com_tablebooking&task=booking.confirmBooking&key=' . self::createConfirmationKeyring($data),
			JUri::root() . 'index.php?option=com_tablebooking&task=booking.cancelBooking&key=' . self::createConfirmationKeyring($data),
			);
		return str_replace($needle, $replacement, $str);
	}


	public static function stripCronTags($text, $input, $params) {
		$relations = array(
			'%date%' => self::showDate($input->bookdate, $params->dateformat),
			'%from%' => self::showTime($input->bookstart, $params->hourformat),
			'%to%' => self::showTime($input->bookend, $params->hourformat),
			'%places%' => $input->places,
			'%name%' => $input->name,
			'%email%' => $input->email,
			'%phone%' => $input->phone
        );
        foreach ($relations as $needle => $replacement) {
			$text = str_replace($needle, $replacement, $text);
        }

        return $text;
	}


	/*
	 * create the confirmation keyring to be attached to the url sent to client
	 * @params [array] $data	reservation details to be used to replace tags
	 *
	 * @return [string]
	 */
	protected static function createConfirmationKeyring($data)
	{
		return $data['id'] . ':' . sha1($data['email']);
	}


    public static function tableNames($restaurant, $tables) {
        $names = '';
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select('name');
        $query->from('#__tbb_tables');
        $query->where('id IN (' . implode(',',$tables) . ')');
        $query->where('restaurant = ' . (int) $restaurant);
        $db->setQuery($query);
        $results = $db->loadColumn();
        if (!empty($results)) {
            $names = implode(',', $results);
        }
        return $names;
    }
}

?>
