<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */


// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class TbUser{

	public static $isAuthorized;
	public static $instance = array();
	public $work;
	public $user;
	public $attached;

	protected function __construct()
	{
		$this->user = JFactory::getUser();
		$this->attached = $this->getAttachedRetaurants();
		$this->work = $this->getWork();
	}


	public static function getInstance( $id = null )
	{
		if (!$id)
		{
			$id = JFactory::getUser()->get('id');
		}

		if (!isset(self::$instance[$id]))
		{
			self::$instance[$id] = new self;
		}
		return self::$instance[$id];
	}


	/*
	 * verify if the user has proper permissions for different areas of management
	 * position 0 - not logged in
	 * position 1 - manager
	 * position 2 - super manager
	 */
	public function isAuthorized($action = 'management')
	{

		if ($this->work->position == 0)
		{
			self::$isAuthorized[$action] = false;
		}
		elseif ( $this->work->position == 2 )
		{
			return self::$isAuthorized[$action] = true;
		}
		else
		{
			if(!isset(self::$isAuthorized[$action]))
			{
				switch($action)
				{
					case 'management':
						self::$isAuthorized[$action] = true;
					break;

					case 'params':
						self::$isAuthorized[$action] = false;
					break;

					default:
						self::$isAuthorized[$action] = false;
					break;
				}
			}
		}
		return self::$isAuthorized[$action];
	}


	protected function getWork()
	{
		$work = new stdclass;
		$work->position = 0;
		$work->restaurant = 0;

		$state = JFactory::getApplication()->getUserState('com_tablebooking.restaurant.selected', null);

		if (empty($state)) {

            if (!empty($this->attached))
    		{
    			$work->restaurant = $this->attached[0]->restaurant;
    			$work->position = $this->attached[0]->position;
    		}
		} else {
			$work->restaurant = TbRestaurant::getInstance($state);
			$work->position = $this->getRestaurantPosition($state);
		}

		return $work;
    }

	protected function getAttachedRetaurants()
	{
    	$attached = array();

        $db = JFactory::getDbo();
		$query = $db->getQuery( true );
		$query->select( $db->quoteName('restaurant') . ',' . $db->quoteName('position') );
		$query->from($db->quoteName('#__tbb_staff'));
		$query->where($db->quoteName('id') . ' = ' . $this->user->get('id') );

		$db->setQuery( $query );
		$result = $db->loadObjectList();

		if (!empty($result)) {
    		foreach ($result as $row) {
				$item = new stdclass;
				$item->position = $row->position;
				$item->restaurant =
					TbRestaurant::getInstance($row->restaurant);
				$attached[] = $item;
    		}
		}

		return $attached;
    }

    protected function getRestaurantPosition($id) {
		$position = 0;

		if (!empty($this->attached)) {
    		foreach ($this->attached as $item) {
				if ($item->restaurant->id == $id) {
					return $item->position;
				}
    		}
		}

		return $position;
    }
}
