<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.modeladmin');

class TableBookingModelBooking extends JModelAdmin
{

	protected $text_prefix = 'COM_TABLEBOOKING_BOOKING';

	public function getTable($type = 'Booking', $prefix = 'TableBookingTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_tablebooking.booking', 'booking', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}
		
		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_tablebooking.edit.booking.data', array());
		
		if (empty($data)) {
			$data = $this->getItem();
		}

		$id = is_array($data) ? 0 : $data->id;
		
		if ( $id )
		{
			$tables = json_decode( $data->table );
			$data->table = implode(',', $tables);
		}
		return $data;
	}

	protected function populateState() 
	{ 	 
		$datephp = JControllerLegacy::getInstance(null)->getModel('Bookings')->getState('filter.bookings.datephp');
		$this->setState('filter.bookings.datephp', $datephp);
		
		$state = JControllerLegacy::getInstance(null)->getModel('Bookings')->getState('filter.bookings.timestamp');
		$this->setState('filter.bookings.timestamp', $state);

		parent::populateState();
	}

	/**
	 * method to return an array containing selected tables for a booking
	 * @return [array] selected tables
	 */
	public function getSelected()
	{
		$data = $this->loadFormData();
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
        $query->select( $db->quoteName('name') );
        $query->from( $db->quoteName('#__tbb_tables') );
        $query->where('id IN(' . $data->table  . ')');

        $db->setQuery($query);
        return $db->loadColumn();
	}
	
	
	/*
	 * clean data before saving
	 * 
	 * @params [array] $data		form data
	 * @return [array] $data		filtered data
	 */
	public function cleanData($data)
	{
		$regexs = array(
				'restaurant' => '/^[0-9]{1,}$/',
				'bookstart' => '/^[0-9]{1,}$/',
				'bookend' => '/^[0-9]{1,}$/',
				'places' => '/^[0-9]{1,}$/',
				'bookdate' => '/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/',
				'table' => '/^[0-9,]*$/',
				'email' => '/^[\w.-]*@[\w.-]*/',
				'phone' => '/[\w\s.\-()+]*/',
				);

		foreach ($data as $key => $value)
		{
			if(isset($regexs[$key]))
			{
				preg_match($regexs[$key],$value,$matches);
				$data[$key] = isset($matches[0]) ? $matches[0] : '';
			}
			else
			{
				$data[$key] = JFilterInput::getInstance()->clean($data[$key],'string');
			}
		}

		return $data;
	}
	
	
	public function changeBookingStatus($id,$email,$newstatus)
	{
		$query = $this->_db->getQuery(true);
		
		$query->update($this->_db->quoteName('#__tbb_bookings'));
		$query->set($this->_db->quoteName('status') . ' = ' . $newstatus);
		$query->where($this->_db->quoteName('id') . ' = ' . (int) $id);
		$query->where('SHA1(email) = ' . $this->_db->quote($this->_db->escape($email)));

		$this->_db->setQuery($query);
		$this->_db->query();
	}
	
	
}
?>
