<?php
/*
 * @component Tablebooking
 * @website : ionutlupu.me
 * @copyright  Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.modeladmin');

class TableBookingModelSearch extends JModelAdmin
{

	protected $text_prefix = 'COM_TABLEBOOKING_BOOKING';

	public function getTable($type = 'Booking', $prefix = 'TableBookingTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_tablebooking.search', 'search', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}

		$nr = $this->getRestaurantsNo();

		if ($nr < 2)
		{
			$form->setFieldAttribute('restaurant', 'class', 'booking-form hidden');
		}
        $this->toggleMandatoryFields($form);

		return $form;
	}

    protected function toggleMandatoryFields(&$form)
    {
        $params = $this->getRestaurant()->getParams();

        if ($params->form_display_phone == 1) {
            $form->setFieldAttribute('phone', 'required', true);
            $form->setFieldAttribute('phone', 'class',
                                        'form-control booking-form');
        }
        if ($params->form_display_phone == 2) {
            $form->setFieldAttribute('phone', 'type', 'hidden');
        }
        if ($params->form_display_comments == 1) {
            $form->setFieldAttribute('comments', 'required', true);
            $form->setFieldAttribute('comments', 'class',
                                            'form-control booking-form');
        }
        if ($params->form_display_comments == 2) {
            $form->setFieldAttribute('comments', 'type', 'hidden');
        }
    }


	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_tablebooking.edit.booking.data', array());
	    $data = (object) $data;
		if (empty($data)) {
			$data = $this->getItem();
		}

        $user = JFactory::getUser();
		if ($user->id)
		{
			$data->name = $user->name;
			$data->email = $user->email;
		}

		return $data;
	}

	public function getRestaurant()
	{
		$restaurant_id = JFactory::getApplication()->getUserState('restaurant.id');

		if (!$restaurant_id)
		{
			$query = $this->_db->getQuery(true);
			$query->select('id');
			$query->from($this->_db->quoteName('#__tbb_restaurants'));
			$query->where($this->_db->quoteName('published') . ' = 1 LIMIT 0,1');

			$this->_db->setQuery($query);
			$restaurant_id = $this->_db->loadResult();
		}

		JFactory::getApplication()->setUserState('restaurant.id', $restaurant_id);

		return TbRestaurant::getInstance($restaurant_id);
	}

	public function setRestaurant($id)
	{
		JFactory::getApplication()->setUserState('restaurant.id', $id);
	}


	public function getRestricted()
	{
		$restaurant = $this->getRestaurant();
		$restrictions = array(
			'restricted' => array(),
			'allowed' => array()
		);

		if (!empty($restaurant->schedule))
		{
			$schedule = unserialize($restaurant->schedule);
			foreach ($schedule as $day => $obj)
			{
				if ($obj->working == 0)
				{
					$restrictions['restricted'][] = (string) $day;
				}
				else
				{
					$restrictions['allowed'][] = (string) $day;
				}
			}
		}

		return $restrictions;
	}


	private function getRestaurantsNo()
	{
		$query = $this->_db->getQuery(true);
		$query->select('COUNT(id)');
		$query->from($this->_db->quoteName('#__tbb_restaurants'));
		$query->where($this->_db->quoteName('published') . ' = 1');

		$this->_db->setQuery($query);
		return $this->_db->loadResult();
	}
}
?>
